

set UserGUIVersion 0

set CustomGUI 0
set Customcfgfile ""

set WriteIF 0x70     ;#   User Register 0
set ReadIF 0x74      ;#   User Register 1

set clock_frequency      105.0

set decimation_factor    4
set shaping_time         6.0
set trapezoid_width      4.4
set peak_separation      8.5
set decay_time_constant  50.0
set peak_sample_time     7.2
set baseline_update_time 2
set baseline_average     4
set baseline_threshold   16384
set mode                 0

set Activate 0

set reg 0     ;# control register
set MWDr$reg 0
for {set reg 16} {$reg <= 31} {incr reg} {set MWDr$reg 0}
for {set i 0} {$i < 11} {incr i} {set MWDp$i 0}

set Reg0x20 0     ;#  CFD Threshold
set Reg0x31 0     ;#  PreTrigger Position

proc VHSADCClient__ReadUserReg {chan reg} {
    global ReadIF boardID

        set regvl [expr ($chan << 28) + ($reg << 16)]        ;#    build data for register
#insert-debug "Writing [format 0x%08x $regvl] to offset $ReadIF"
        VHSADCClient__Vhsadac16WriteFpgaReg $boardID $ReadIF $regvl        ;#    write to user register
        VHSADCClient__Vhsadac16WriteFpgaReg $boardID $ReadIF $regvl        ;#    write to user register
        set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID $ReadIF]  ;# user register
#insert-debug "Read [format 0x%04x $rc] from offset $ReadIF"

    return $rc
}

proc VHSADCClient__WriteUserReg {chan reg vl} {
    global WriteIF boardID

        set regvl [expr ($chan << 28) + ($reg << 16) + $vl]        ;#    build data for register
#insert-debug "Writing [format 0x%08x $regvl] to offset $WriteIF"
        VHSADCClient__Vhsadac16WriteFpgaReg $boardID $WriteIF $regvl        ;#    write to user register
}

proc do_user_registers {} {
    global UserGUIVersion

     make_UserGUI_$UserGUIVersion
}

proc make_UserGUI_0 {} {

#    make default User FPGA register interface

  :frame "[self frame-name].user" :label "User defined control registers" :resize F :transient T

   :panel P :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 10

      for {set i 0} {$i <= 11} {incr i} {
          :text USERVL$i :label Reg$i :valuex 6c :w 10
          :next-row T
      }

      :button USERDONE  :label "Done" :command "frame-quit [self frame-name].user"
      :button USERUPDTE :label "Update" :command update_user_0

  :show T

   update_user_0
}

proc update_user_0 {} {
    global boardID

    for {set i 0} {$i <= 3} {incr i} {
        set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x70 + ($i*4)]]  ;# user register
        :modify USERVL$i :value [format 0x%08x $rc]
    }

    for {set i 4} {$i <= 11} {incr i} {
        set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID [expr 0x100 + (($i-4)*4)]]  ;# user register
        :modify USERVL$i :value [format 0x%08x $rc]
    }
}

proc click-uservl {ch vl} {
    global boardID

   if {$ch <= 3} {
       VHSADCClient__Vhsadac16WriteFpgaReg $boardID [expr 0x70 + ($ch*4)]]  $vl
   } else {
       VHSADCClient__Vhsadac16WriteFpgaReg $boardID [expr 0x100 + (($ch-4)*4)]]  $vl
   }

   update_user_0
}

proc make_UserGUI_1 {}  {
   global  numchans chanlist chan
   global allmodules allchans
   global Options Customcfgfile
   global Usercfg UserReg

   set Customcfgfile $Options(FpgaRegisters)

    set z [catch {open $Customcfgfile r} f]
    if {$z != 0} then {
          midas-warning "Cannot read file $Customcfgfile\n$f"
          return
    }
    set Usercfg [split [read $f] \n]
    close $f

#    process configuration file

    set title "User Acquisition Registers"

    foreach item $Usercfg {
       if {[string range $item 0 0] == "#"} {continue}
       if {[first $item] == "name"} {set title [second $item]; break}
    }

  :frame "[self frame-name].user" :label "$title" :show-footer T  :resize F

   :panel MODULE  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10

     :number CHAN-NUM  :label "Channel number:" :min 0 :max [expr $numchans-1] :w 5 :value $chan
     :menubutton CHAN-SEL :label "select:" :strings $chanlist
     :hskip 50
     :checkbox ALL-MODULES :label "Act on ALL modules?"  :value $allmodules
     :checkbox ALL-CHANS   :label "Act on ALL channels?" :value $allchans

   :panel REGISTERS  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10

    set UserReg {}

    foreach item $Usercfg {
       if {[string range $item 0 0] == "#"} {continue}
       if {[first $item] == "name"} {continue}
       switch [first $item] {
           choice  {
               set ID [second [split [third $item] =]]
               lappend UserReg $ID
               set options [lreplace $item 0 2]
               set menu {}
               foreach element $options {
                   lappend menu [second [split $element :]]
               }
               :choice USER$ID :label "[second $item]" :strings $menu
               :next-row T
           }
           slider {
               set ID [second [split [third $item] =]]
               lappend UserReg $ID
               set Units [second [split [lindex $item 3] =]]
               set Range [second [split [lindex $item 4] =]]
               set min [first [split $Range ,]]
               set max [second [split $Range ,]]
               :slider USER$ID :label "[second $item]" :min [second [split $min :]] :max [second [split $max :]]
               :next-row T
           }
           value {
               set ID [second [split [third $item] =]]
               lappend UserReg $ID
               :text USER$ID :label "[second $item]"
               :next-row T
           }
           default {}
       }
    }

  :show T

   update_user_1
}

proc update_user_1 {} {
   global boardID chan chanlist
   global Usercfg UserReg
   global ReadIF

   foreach id $UserReg {

        set regvl [expr ($chan << 28) + ($id << 16)]              ;#    build data for register
#insert-debug "Writing [format 0x%08x $regvl] to offset $ReadIF"
        VHSADCClient__Vhsadac16WriteFpgaReg $boardID $ReadIF $regvl ;#    write to user register

        set rc 25
        set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID $ReadIF]  ;#  read user register
#insert-debug "Read of offset $ReadIF returned [format 0x%08x $rc]"
        :modify USER$id :value [expr $rc & 0x0000ffff]
   }
}

proc click_user_1 {id vl} {
    global boardID boardlist chan chanlist
    global allmodules allchans

#insert-debug "click with $id $vl"

    if {$allmodules == 0} {

        if {$allchans == 0} {
            VHSADCClient__WriteUserReg $chan $id $vl
        } else {
            foreach ch $chanlist {
                VHSADCClient__WriteUserReg $ch $id $vl
            }
        }
    } else {
        set boardID_SAV $boardID
        foreach boardID $boardlist {

            if {$allchans == 0} {
                VHSADCClient__WriteUserReg $chan $id $vl
            } else {
                foreach ch $chanlist {
                    VHSADCClient__WriteUserReg $ch $id $vl
                }
            }
        }
        set boardID $boardID_SAV
    }
}

proc click-select-chan {vl} {
    global chan UserGUIVersion
    global NumChan boardID

    if {$vl >= $NumChan($boardID)} {
         midas-warning "Channel not supported by FPGA firmware"
         set chan 0
    } else {
         set chan $vl
    }

    :modify CHAN-NUM :value $chan

    update_user_$UserGUIVersion
}


set user10 0
set user11 0
set user12 0
set user20 0
set user21 0
set user22 0


for {set i 0} {$i <= 31} {incr i} {set RegMask($i) 0x0000ffff}
set RegMask(16) 255
set RegMask(17) 255
set RegMask(20) 255
set RegMask(21) 255
set RegMask(24) 255
set RegMask(25) 31
set RegMask(26) 3


proc make_UserGUI_2 {} {
    global user10 user11 user12
    global user20 user21 user22
    global numchans chan chanlist allchans allmodules

#    GUI interface for MPD firmware

    set menu1 {}
    for {set i 0} {$i <= 15} {incr i} {lappend menu1 $i}
    set menu2 {}
    for {set i 0} {$i <= 111} {incr i} {lappend menu2 [format 0x%02x $i]}
    lappend menu2 0x80; lappend menu2 0x81; lappend menu2 0x82;
 
  :frame "[self frame-name].user" :label "Firmware Parameter Control" :resize F :transient T

   :panel MODULE  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10

     :text USER20   :label "VHS-ADC Board Selection" :inactive T :w 3

     :number CHAN-NUM  :label "Channel number:" :min 0 :max [expr $numchans-1] :w 5 :value $chan
     :menubutton CHAN-SEL :label "select:" :strings $chanlist
     :hskip 40
     :checkbox ALL-MODULES :label "Act on ALL modules?"  :value $allmodules
     :checkbox ALL-CHANS   :label "Act on ALL channels?" :value $allchans

   :container C0 :layout T

   :panel P0L :layout L :padx 60 :pady 0
     :layout V; :row-gap 10; :col-gap 10

      :choice USER2CW :label "MWD Control Word" :exclusive F :valuex 20c :layout V :w 40  \
          :strings [list "Disable MWD" "Disable Baseline subtraction" "Data Shift (LSB)" "Data Shift (MSB)" "Trigger Polarity: 0 = +ve, 1 = -ve" \
               "Test Select bit 0" "Test Select bit 1" "Test Select bit 2"   \
               "Data Select (LSB)" "Data Select (MSB)" GPIO_OE "Swap (=1 inverts order of DAC data)" "Trigger: 0=external, 1=internal (CFD)"]

   :panel P0R :layout R :padx 60 :pady 0
     :layout V; :row-gap 10; :col-gap 10

      :choice USER2SW :label "MWD Status Word"  :exclusive F :valuex 20c :layout V :w 40 :inactive T  \
          :strings [list myready mwd_pileup watch_pu decclk_locked watchdog_happened received_triiger started_reading_circbuf finished_reading_circbuf got_energy]

       :vskip 25
       :choice XXX :label "legend" :layout V :exclusive F :w 40 :strings [list "this bit is 0/off/false" "this bit is 1/on/true"] :value 2 :inactive T

   :container end

   :container C1 :layout B

   :panel P1L :layout L :padx 5 :pady 5
     :layout V; :row-gap 10; :col-gap 10

      :text CLOCK_FREQUENCY      :label "Clock Frequency"  :w 8 :valuex 30c
      :text SHAPING_TIME         :label "Shaping (rise time + flat top)"  :w 8 :valuex 30c
      :text TRAPEZOID_WIDTH      :label "Rise Time"  :w 8 :valuex 30c
      :text PEAK_SAMPLE_TIME     :label "Peak Sample"  :w 8 :valuex 30c
      :text PEAK_SEPARATION      :label "Peak Separation"  :w 8 :valuex 30c
      :text BASELINE_AVERAGE     :label "Baseline Average"  :w 8 :valuex 30c
      :text BASELINE_UPDATE_TIME :label "Baseline Update Clocks"  :w 8 :valuex 30c
      :text BASELINE_THRESHOLD   :label "Baseline Threshold"  :w 8 :valuex 30c
      :text DECAY_TIME_CONSTANT  :label "Decay Time Constant"  :w 8 :valuex 30c
      :text DECIMATION_FACTOR    :label "Decimation"  :w 8 :valuex 30c
      :text MODE                 :label "Mode"  :w 8 :valuex 30c

      :vskip 25
      :text USER2CFD             :label "CFD Threshold"  :w 8 :valuex 30c

   :panel P1M :layout L :padx 5 :pady 5
     :layout V; :row-gap 10; :col-gap 10

      :text MWDR16  :label "MWDR16"  :w 12 :valuex 10c
      :text MWDR17  :label "MWDR17"  :w 12 :valuex 10c
      :text MWDR18  :label "MWDR18"  :w 12 :valuex 10c
      :text MWDR19  :label "MWDR19"  :w 12 :valuex 10c
      :text MWDR20  :label "MWDR20"  :w 12 :valuex 10c
      :text MWDR21  :label "MWDR21"  :w 12 :valuex 10c
      :text MWDR22  :label "MWDR22"  :w 16 :valuex 10c
      :text MWDR23  :label "MWDR23"  :w 16 :valuex 10c
      :text MWDR25  :label "MWDR25"  :w 12 :valuex 10c
      :text MWDR26  :label "MWDR26"  :w 12 :valuex 10c
      :text MWDR27  :label "MWDR27"  :w 12 :valuex 10c
      :text MWDR28  :label "MWDR28"  :w 12 :valuex 10c
      :text MWDR30  :label "MWDR30"  :w 12 :valuex 10c

   :panel P1R :layout R :padx 5 :pady 5
     :layout V; :row-gap 30;

      :vskip 100
      :button USER2ACT :label "Activate MWD parameter changes" 

      :vskip 100
      :text USER2PRETRIG   :label "PreTrigger (card)"  :w 8


    :container end

   :panel P1 :layout B :padx 5 :pady 5   :after FOOTER
     :layout H; :row-gap 10; :col-gap 10

      :text USER21          :label "User Register 1" :inactive T :w 12
      :choice-stack USER210 :label channel :value $user10 :strings $menu1 :valuex 16c
      :choice-stack USER211 :label "register address" :value $user11 :strings $menu2 :valuex 16c
      :text USER212         :label "register value" :w 8 :value $user12 :valuex 16c
      :button USER213       :label "Write"
      :next-row T

      :text USER22          :label "User Register 2" :inactive T :w 12
      :choice-stack USER220 :label channel :value $user20 :strings $menu1 :valuex 16c
      :choice-stack USER221 :label "register address" :value $user21 :strings $menu2 :valuex 16c
      :text USER222         :label "register value" :w 8 :value $user22 :valuex 16c
      :button USER223       :label "Read"
      :next-row T

   :panel BUTTONS :layout B :padx 5 :pady 5  :after FOOTER
     :layout H; :row-gap 10; :col-gap 10

      :button USERDONE  :label "Done" :command "frame-quit [self frame-name].user"
      :button USERUPDTE :label "Update" :command update_user_2
      :hskip 100
      :message WARNING     :label  "Warning: Access to MWD parameters is only reliable when not GOING" :bold T

  :show T

   update_user_2
}

proc Read_MWD_Registers {} {
    global boardID chan
    global RegMask

    global MWDr0 Reg0x20 Reg0x31
    for {set reg 16} {$reg <= 31} {incr reg} {global MWDr$reg}

    set-footer "Reading MWD parameters for board $boardID channel $chan"

    set reg 0
    set rc [VHSADCClient__ReadUserReg $chan $reg]
    set MWDr$reg [expr $rc & $RegMask($reg)]

    for {set reg 16} {$reg <= 31} {incr reg} {
        set rc [VHSADCClient__ReadUserReg $chan $reg]
        set MWDr$reg [expr $rc & $RegMask($reg)]
    }

    set-footer "Adding CFD Threshold and PreTrigger Position"

    set reg 0x20
    set rc [VHSADCClient__ReadUserReg $chan $reg]
    set Reg$reg [expr $rc & 0x0000ffff]

    set reg 0x31
    set rc [VHSADCClient__ReadUserReg 0 $reg]
    set Reg$reg [expr $rc & 0x0000ffff]

    clear-footer 

}

proc Set_MWD_Registers {} {
    global boardID chanlist NumChan

    set-footer "Enabling MWD parameters for board $boardID"

    foreach chan $chanlist {

        if {$chan >= $NumChan($boardID)} {break}

        set rc [VHSADCClient__ReadUserReg $chan 0]
        VHSADCClient__WriteUserReg $chan 0 [expr $rc | 1]   ;# disable MWD
        VHSADCClient__WriteUserReg $chan 0 $rc
    }

    clear-footer

    update_mwd_parameters
    apply_mwd_parameters

    update_user_2   ;# if needed

}

proc update_user_2 {} {
    global boardID WriteIF ReadIF
    global chan

    for {set reg 16} {$reg <= 31} {incr reg} {global MWDr$reg}
    global Reg0x20 Reg0x31

    :modify [self frame-name].user :busy T

    :modify USER20  :inactive F :value $boardID :inactive T

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID $WriteIF]  ;# user register
    :modify USER21  :inactive F :value [format 0x%08x $rc] :inactive T

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID $ReadIF]  ;# user register
    :modify USER22  :inactive F :value [format 0x%08x $rc] :inactive T

    set reg 0                                               ;# Control Word
        set-footer [self frame-name].user "Reading register $reg"
        set rc [VHSADCClient__ReadUserReg $chan $reg]
        :modify USER2CW  :value [expr $rc & 0x00001fff]

    set reg 1                                               ;# Status Word
        set-footer [self frame-name].user "Reading register $reg"
        set rc [VHSADCClient__ReadUserReg $chan $reg]
        :modify USER2SW  :inactive F :value [expr $rc & 0x000001ff] :inactive T

    set-footer [self frame-name].user "Reading MWD registers"
    Read_MWD_Registers
    clear-footer [self frame-name].user

    foreach reg [list 16 17 18 19 20 21 22 25 26 27] {
        :modify MWDR$reg :value [format "%d (0x%04x)" [set MWDr$reg] [set MWDr$reg]]
    }

    :modify MWDR23 :value [format 0x%08x [expr ($MWDr24 << 16) + $MWDr23]]
    :modify MWDR28 :value [format 0x%08x [expr ($MWDr29 << 16) + $MWDr28]]
    :modify MWDR30 :value [format 0x%08x [expr ($MWDr31 << 16) + $MWDr30]]

    :modify USER2CFD  :value [expr $Reg0x20 & 0x0000ffff]
    :modify USER2PRETRIG  :value [expr $Reg0x31 & 0x0000ffff]

    update_mwd_parameters

    :modify [self frame-name].user :busy F
}

proc click_user_2 {id vl index} {
    global boardID boardlist chan chanlist NumChan
    global allmodules allchans
    global Activate

#insert-debug "click_user_2 with $id $vl $index"

    set chan_SAV $chan
    set boardID_SAV $boardID

   switch $id {
     10 -
     11 -
     12 -
     13 -
     20 -
     21 -
     22 -
     23 {
          :modify [self frame-name].user :busy T
          do_click_user_2 $chan $id $vl $index
          :modify [self frame-name].user :busy F
          return
     }
   }

    set state [inform sys inquire readoutstate]
    if {$state != 0} {
        midas-warning "You can only do this reliably when STOPped"
        return
    }

    :modify [self frame-name].user :busy T

    if {$allmodules == 0} {

        if {$allchans == 0} {
            do_click_user_2 $chan $id $vl $index
        } else {
            foreach ch $chanlist {
                 if {$ch >= $NumChan($boardID)} {break}
                 set-footer [self frame-name].user "Setting board $boardID; channel $ch"
                 do_click_user_2 $ch $id $vl $index
            }
        }
    } else {
        foreach boardID $boardlist {

            if {$allchans == 0} {
                set-footer [self frame-name].user "Setting board $boardID; channel $chan"
                do_click_user_2 $chan $id $vl $index
            } else {
                foreach ch $chanlist {
                    if {$ch >= $NumChan($boardID)} {break}
                    set-footer [self frame-name].user "Setting board $boardID; channel $ch"
                    do_click_user_2 $ch $id $vl $index
                }
            }
        }
    }

    set boardID $boardID_SAV
    set chan $chan_SAV

    :modify [self frame-name].user :busy F
    clear-footer [self frame-name].user

    if {$allchans != 0} {click-select-chan $chan}

    if {$Activate == 1} {set-footer [self frame-name].user "Remember to Activate MWD parameters changes when finished"}
}

proc do_click_user_2 {chan id vl index} {
    global boardID WriteIF ReadIF
    global user10 user11 user12
    global user20 user21 user22
    global RegMask
    global Activate
    for {set reg 16} {$reg <= 31} {incr reg} {global MWDr$reg}

#insert-debug "do_click_user_2 with $boardID $chan $id $vl $index"

    switch $id {
        10 {set user10 $index}
        11 {set user11 $index}
        12 {set user12 $vl}
        13 {                          ;# click on button USER213   (Write User Register 1)
            VHSADCClient__WriteUserReg $user10 $user11 $user12
        }
        20 {set user20 $index}
        21 {set user21 $index}
        22 {set user22 $vl}
        23 {                          ;# click on button USER223   (Read User Register 2)
            set rc [VHSADCClient__ReadUserReg $user20 $user21]
            :modify USER22  :inactive F :value [format 0x%08x $rc] :inactive T
            if {[expr ($rc >> 28) & 15] == $user20 && [expr ($rc >> 16) & 0x0fff] == $user21} {
                :modify USER222 :value [format 0x%04x [expr $rc & 0x0000ffff]]
            } else {
                :modify USER222 :value "******"
            }
        }
        R16 -
        R17 -
        R18 -
        R19 -
        R20 -
        R21 -
        R22 -
        R25 -
        R26 {
            set reg [string range $id 1 end]
            VHSADCClient__WriteUserReg $chan $reg $vl
            set rc [VHSADCClient__ReadUserReg $chan $reg]
            set MWDr$reg [expr $rc & $RegMask($reg)]
            :modify MWDR$reg :value [format "%d (0x%04x)" [set MWDr$reg] [set MWDr$reg]]
            set Activate 1
        }
        R23 {
            VHSADCClient__WriteUserReg $chan 23 [expr $vl & 0x0000ffff]
            VHSADCClient__WriteUserReg $chan 24 [expr ($vl >> 16) & 0x000000ff]
            set MWDr23 [expr [VHSADCClient__ReadUserReg $chan 23] & $RegMask(23)]
            set MWDr24 [expr [VHSADCClient__ReadUserReg $chan 24] & $RegMask(24)]
            :modify MWDR23 :value [format 0x%08x [expr ($MWDr24 << 16) + $MWDr23]]
            set Activate 1
        }
        CW {
            VHSADCClient__WriteUserReg $chan 0 [expr $vl & 0x0000ffff]
            set rc [VHSADCClient__ReadUserReg $chan 0]
            :modify USER2CW :value [format 0x%04x [expr $rc & 0x0000ffff]]
            set Activate 1
        }
        CFD {
            VHSADCClient__WriteUserReg $chan 0x20 [expr $vl & 0x0000ffff]
            set rc [VHSADCClient__ReadUserReg $chan 0x20]
            :modify USER2CFD :value [expr $rc & 0x0000ffff]
            set Activate 1
        }
        PRETRIG {
            set reg 0x31
            VHSADCClient__WriteUserReg 0 $reg [expr $vl & 0x0000ffff]
            set rc [VHSADCClient__ReadUserReg 0 $reg]
            :modify USER2PRETRIG :value [expr $rc & 0x0000ffff]
            set Activate 1
        }
        ACT {
            set rc [VHSADCClient__ReadUserReg $chan 0]
            VHSADCClient__WriteUserReg $chan 0 [expr $rc | 1]   ;# disable MWD
            VHSADCClient__WriteUserReg $chan 0 $rc
            set rc [VHSADCClient__ReadUserReg $chan 0]
            :modify USER2CW :value [format 0x%04x [expr $rc & 0x0000ffff]]
            set Activate 0
        }
    }

    update_mwd_parameters
}

proc click-user2cfd {v} {

    click_user_2 CFD $v 0

}

proc click-user2pretrig {v} {

    click_user_2 PRETRIG $v 0

}

proc click-clock_frequency  {v} {
    global clock_frequency

    set clock_frequency $v
    apply_mwd_parameters
}

proc click-decimation_factor  {v} {
    global decimation_factor
    global MWDp9

    set decimation_factor $v
    apply_mwd_parameters
    click_user_2 R25 $MWDp9 0
}

proc click-shaping_time  {v} {
    global shaping_time
    global MWDp0 MWDp7

    set shaping_time $v
    apply_mwd_parameters
    click_user_2 R16 $MWDp0 0
    click_user_2 R23 $MWDp7 0
}

proc click-trapezoid_width   {v} {
    global trapezoid_width
    global MWDp1

    set trapezoid_width $v
    apply_mwd_parameters
    click_user_2 R17 $MWDp1 0
}

proc click-peak_separation  {v} {
    global peak_separation
    global MWDp3

    set peak_separation $v
    apply_mwd_parameters
    click_user_2 R19 $MWDp3 0
}

proc click-decay_time_constant  {v} {
    global decay_time_constant
    global MWDp7

    set decay_time_constant $v
    apply_mwd_parameters
    click_user_2 R23 $MWDp7 0
}

proc click-peak_sample_time   {v} {
    global peak_sample_time
    global MWDp2

    set peak_sample_time $v
    apply_mwd_parameters
    click_user_2 R18 $MWDp2 0
}

proc click-baseline_update_time   {v} {
    global baseline_update_time
    global MWDp5

    set baseline_update_time $v
    apply_mwd_parameters
    click_user_2 R21 $MWDp5 0
}

proc click-baseline_average {v} {
    global baseline_average
    global MWDp4

    set baseline_average $v
    apply_mwd_parameters
    click_user_2 R20 $MWDp4 0
}

proc click-baseline_threshold {v} {
    global baseline_threshold
    global MWDp6

    set baseline_threshold $v
    apply_mwd_parameters
    click_user_2 R22 $MWDp6 0
}

proc click-mode {v} {
    global mode
    global MWDp10

    set mode $v
    apply_mwd_parameters
    click_user_2 R26 $MWDp10 0
}

proc apply_mwd_parameters {} {
    global clock_frequency decimation_factor shaping_time  trapezoid_width
    global peak_separation decay_time_constant peak_sample_time
    global baseline_average  baseline_update_time baseline_threshold mode

    for {set i 0} {$i < 11} {incr i} { global MWDp$i }

    set clock_width [expr 1000000000 / ( [set clock_frequency] * 1000000 ) ]
    set ddt [expr $clock_width * 0.001 * $decimation_factor ]
    set dt [expr $clock_width * 0.001 ]
    set Mbydt [expr $shaping_time / $ddt]

    set MWDp0 [expr round(256 - $Mbydt +1)]                             ;# R16
    set MWDp1 [expr 256 - round($trapezoid_width / $ddt ) -1]           ;# R17
    set MWDp2 [expr int($peak_sample_time / $dt)]                       ;# R18
    set MWDp3 [expr int( $peak_separation / $dt ) ]                     ;# R19
    set MWDp4 $baseline_average                                         ;# R20
    set MWDp5 [expr round($baseline_update_time / $ddt)]                ;# R21
    set MWDp6 $baseline_threshold                                       ;# R22

    set MWDp7 [expr round(16777216 * (1 - exp([expr - $shaping_time / $decay_time_constant])) / $Mbydt)]    ;# R23

    set MWDp9 [expr 32 - $decimation_factor +1]                         ;# R25
    set MWDp10 $mode                                                    ;# R26

}

proc update_mwd_parameters {} {
    global clock_frequency decimation_factor shaping_time  trapezoid_width
    global peak_separation decay_time_constant peak_sample_time 
    global baseline_average baseline_update_time baseline_threshold mode
    global Activate

    for {set reg 16} {$reg <= 31} {incr reg} {global MWDr$reg}

    set clock_width [expr 1000000000 / ( [set clock_frequency] * 1000000 ) ]
    set decimation_factor [expr 33 - $MWDr25]
    set ddt [expr $clock_width * 0.001 * $decimation_factor ]
    set dt [expr $clock_width * 0.001 ]
    set shaping_time [expr (257 - $MWDr16) * $ddt]
    set Mbydt [expr $shaping_time / $ddt]

#    set shaping_time [expr 257 - $MWDr16]
    set trapezoid_width [expr (255 - $MWDr17) * $ddt]
    set peak_sample_time [expr $MWDr18 * $dt]
    set peak_separation [expr $MWDr19 * $dt]
    set baseline_average $MWDr20
    set baseline_update_time [expr round($MWDr21 * $ddt)]
    set baseline_threshold $MWDr22
    set p1 [expr ($MWDr23*$Mbydt)/16777216]
    set p2 [expr 1/(1-$p1)]
    set decay_time_constant [expr $shaping_time/log($p2)]
#    set decimation_factor [expr 33 - $MWDr25]
    set mode $MWDr26

    :modify CLOCK_FREQUENCY      :value [string range $clock_frequency 0 5]
    :modify DECIMATION_FACTOR    :value [string range $decimation_factor 0 5]
    :modify SHAPING_TIME         :value [string range $shaping_time 0 5]
    :modify TRAPEZOID_WIDTH      :value [string range $trapezoid_width 0 5]
    :modify PEAK_SEPARATION      :value [string range $peak_separation 0 5]
    :modify DECAY_TIME_CONSTANT  :value [string range $decay_time_constant 0 5]
    :modify PEAK_SAMPLE_TIME     :value [string range $peak_sample_time 0 5]
    :modify BASELINE_UPDATE_TIME :value [string range $baseline_update_time 0 5]
    :modify BASELINE_AVERAGE     :value [string range $baseline_average 0 5]
    :modify BASELINE_THRESHOLD   :value [string range $baseline_threshold 0 5]
    :modify MODE                 :value [string range $mode 0 5]

    if {$Activate == 1} {set-footer [self frame-name].user "Remember to Activate MWD parameters changes when finished"}

}
